#include <iostream>
#include <cstdlib>
#include <time.h>
#include <cstring>
#include <random>

#ifdef _OPENMP
#include <omp.h>
#endif

#include "timer.hxx"
#include "results.hxx"
#include "binary.hxx"

int main(int argc, char **argv) {

  Timer T_total, T_init, T_calcul;
  T_total.start();
  T_init.start();
  
  double moy, var;
  int i, n;
  double *u;
  readVector("data.bin", u, n);

  int nThreads;

  #ifdef _OPENMP
  #pragma omp parallel
  {
    #pragma omp master
    nThreads = omp_get_num_threads();
  }
  #else
    nThreads = 1;
  #endif

  std::cerr << "version 1, " << nThreads << " thread(s)" << std::endl;
  std::cerr << "n = " << n << " u0 = " << u[0] << std::endl;

  T_init.stop();
  T_calcul.start();

  double * s_partiel, * s2_partiel;
  int iTh;

  s_partiel = new double[nThreads];
  s2_partiel = new double[nThreads];
  for (iTh=0; iTh < nThreads; iTh++) {
    s_partiel[iTh] = 0.0;
    s2_partiel[iTh] = 0.0;
  }

  #pragma omp parallel for default (shared) private (iTh)
  for (i=0; i<n ; i++) {

#ifdef _OPENMP
    iTh = omp_get_thread_num();
#else
    iTh = 0;
#endif
    s_partiel[iTh] += u[i];
    s2_partiel[iTh] += u[i]*u[i];
  }

  delete [] u;
  double s = 0.0, s2 = 0.0;
  double * p = s_partiel, *p2 = s2_partiel;

  for (iTh=0; iTh < nThreads; iTh++) {
    s += *p++;
    s2 +=*p2++;
  }

  delete [] s_partiel;
  delete [] s2_partiel;

  moy = s / n;
  var = s2/n - moy*moy;

  T_calcul.stop();

  T_total.stop();

  writeResults("version1", n, nThreads, moy, var, 
                T_init.elapsed(), T_calcul.elapsed(), T_total.elapsed());
  return 0;
}
