#include <iostream>
#include <cstdlib>
#include <time.h>
#ifdef _OPENMP
#include <omp.h>
#endif
#include <cstring>

#include "timer.hxx"
#include "results.hxx"
#include "binary.hxx"

int main(int argc, char **argv) {

  Timer T_total, T_init, T_calcul;
  T_total.start();
  T_init.start();
  
  double moy, var;
  int offset = argc > 1 ? strtol(argv[1], NULL, 10) : 30;
  int n;
  double * u;
  readVector("data.bin", u, n);

  int nThreads;

  #ifdef _OPENMP
  #pragma omp parallel
  {
    #pragma omp master
    nThreads = omp_get_num_threads();
  }
  #else
    nThreads = 1;
  #endif

  std::cerr << "version 2, " << nThreads << " thread(s)" << std::endl;
  std::cerr << "n = " << n << " u0 = " << u[0] << std::endl;

  T_init.stop();
  T_calcul.start();

  double * s_partiel, * s2_partiel;
  int iTh;

  s_partiel = new double[nThreads * offset];
  s2_partiel = new double[nThreads * offset];
  for (iTh=0; iTh < nThreads; iTh++) {
    s_partiel[iTh * offset] = 0.0;
    s2_partiel[iTh * offset] = 0.0;
  }

#pragma omp parallel default (shared)
  {
    int iTh, i;
  #pragma omp for
  for (i=0; i<n ; i++) {

#ifdef _OPENMP
      iTh = omp_get_thread_num();
#else
      iTh = 0;
#endif
      s_partiel[iTh * offset] += u[i];
      s2_partiel[iTh * offset] += u[i]*u[i];
    }
  }
  delete [] u;

  double s = 0.0, s2 = 0.0;
  double * p = s_partiel, *p2 = s2_partiel;

  for (iTh=0; iTh < nThreads; iTh++, p += offset, p2 += offset) {
    s += *p;
    s2 +=*p2;
  }

  delete [] s_partiel;
  delete [] s2_partiel;

  moy = s / n;
  var = s2/n - moy*moy;

  T_calcul.stop();

  T_total.stop();

  writeResults("version2", n, nThreads, moy, var, 
                T_init.elapsed(), T_calcul.elapsed(), T_total.elapsed());
}
